-- -----------------------------------------------------------------------------------
-- 画面部品
-- -----------------------------------------------------------------------------------

UPDATE TM_CDB_VIEW_PART a 
SET seqno = newseqno 
FROM 
(
    SELECT 
        x.sid_corporate,
        x.table_id,
        x.id,
        x.view_id,
        y.holder_field_fid,
        ROW_NUMBER() OVER (PARTITION BY x.sid_corporate, x.table_id, x.view_id ORDER BY x.seqno ASC, x.id ASC) AS newseqno
    FROM TM_CDB_VIEW_PART x 
        INNER JOIN tm_cdb_view y ON x.sid_corporate=y.sid_corporate AND x.table_id=y.table_id AND x.view_id=y.id 
) b 
WHERE 
    a.sid_corporate=b.sid_corporate AND 
    a.table_id=b.table_id AND 
    a.view_id=b.view_id AND 
    a.id=b.id AND
    b.holder_field_fid IS NULL
;

-- -----------------------------------------------------------------------------------
-- 画面の全削除で削除された子画面を修復
-- -----------------------------------------------------------------------------------
delete from tm_cdb_table where table_id < 0
;
-- 子画面を再登録する（表部品、参照データ一覧、ルックアップ選択ダイアログ画面）
insert into tm_cdb_view 
  (sid_corporate, table_id, id, seqno, type_, updated_by, cretime, updtime, enabled_, holder_field_tid, holder_field_fid, lastid, sort_field_cid, sort_field_tid, sort_field_fid, sort_order)
  select 
    h.sid_corporate as sid_corporate, 
    trunc(h.table_id, -3) as table_id, 
    ROW_NUMBER() OVER(partition by h.sid_corporate, trunc(h.table_id, -3) order by h.sid_corporate, h.table_id, h.id asc) + 1 as id,
    ROW_NUMBER() OVER(partition by h.sid_corporate, trunc(h.table_id, -3) order by h.sid_corporate, h.table_id, h.id asc) + 1 as seqno,
    'table' as type_, 
    1 as updated_by, 
    h.cretime as cretime,
    h.updtime as updtime,
    '0' as enabled_,
    h.table_id as holder_field_tid,
    h.id as holder_field_fid,
    case h.rel_lookup
      when '1' then 1 end as lastid, 
    NULL as sort_field_cid, 
    NULL as sort_field_tid, 
    NULL as sort_field_fid, 
    case h.rel_lookup
      when '1' then 'asc' end as sort_order
  from
    tm_cdb_table_field h
  where 
    (type_ in ('input_list', 'rel_list') or rel_lookup='1') and 
    not exists(select 1 from tm_cdb_view where sid_corporate=h.sid_corporate and holder_field_tid=h.table_id and holder_field_fid=h.id)
;
-- 表部品の子部品を再登録する
insert into tm_cdb_view_part
  (sid_corporate, table_id, view_id, id, seqno, type_, field_tid, field_fid, class_name, editable_, cretime, updtime)
  select
    v.sid_corporate as sid_corporate,
    v.table_id as table_id,
    v.id  as view_id,
    f.id as id,
    f.id as seqno,
    'field' as type_,
    f.table_id  as field_tid,
    f.id as field_fid,
    case f.type_ 
      when 'files' then 'cdb-part-files-list' end as class_name,
    case f.type_ 
      when 'expression' then '0' 
      when 'rel_field' then '0' 
      else '1' end as editable_,
    f.cretime as cretime, 
    f.updtime as updtime
  from 
    (select * from tm_cdb_table_field where type_='input_list') h
    join tm_cdb_table_field f on (f.sid_corporate=h.sid_corporate and f.table_id=h.rel_table_id) 
    join tm_cdb_view v on (v.holder_field_tid=h.table_id and v.holder_field_fid=h.id)
  where 
    f.system_='0' and
    not exists (select * from tm_cdb_view_part where sid_corporate=v.sid_corporate and table_id=v.table_id and view_id=v.id and field_tid=f.table_id and field_fid=f.id)
;

-- -----------------------------------------------------------------------------------
-- 添付ファイル部品初期値
-- -----------------------------------------------------------------------------------
with recursive r(fld_cid, fld_tid, fld_fid, fld_rtid, fld_rfid, fld_type, act_type, level) as (
  select sid_corporate, table_id, id, rel_table_id, rel_field_id, type_, type_, 0 from tm_cdb_table_field 
    union all
  select fld_cid, fld_tid, fld_fid, rel_table_id, rel_field_id, fld_type, type_, level+1 from tm_cdb_table_field f, 
    r where f.sid_corporate = r.fld_cid and f.table_id = r.fld_rtid and f.id = r.fld_rfid and r.fld_type = 'rel_field'
)
update
  tm_cdb_view_part
set
  class_name = trim(coalesce(p.class_name, '') || ' ' || 
    case
      when v.type_ = 'card' then 'cdb-part-files-list'
      when v.holder_field_type = 'input_list' then 'cdb-part-files-list'
      when a.fld_type = 'rel_field' then 'cdb-part-files-list'
      else 'cdb-part-files-exists'
    end)
from 
  (select * from tm_cdb_view_part where type_='field' and coalesce(class_name,'') not like '%cdb-part-files-%') p
  join (
    select v_.*, f_.type_ as holder_field_type from tm_cdb_view v_ left join tm_cdb_table_field f_ on (v_.holder_field_tid=f_.table_id and v_.holder_field_fid=f_.id)
  ) v on (p.sid_corporate=v.sid_corporate and p.table_id=v.table_id and p.view_id=v.id)
  join (
    select fld_cid, fld_tid, fld_fid, fld_type, act_type from (
      select fld_cid, fld_tid, fld_fid, fld_type, act_type, level, row_number() over (partition by fld_cid, fld_tid, fld_fid order by level desc) as rlevel from r
    ) x where x.rlevel=1 and x.act_type='files'
  ) a on (p.sid_corporate=a.fld_cid and p.field_tid=a.fld_tid and p.field_fid=a.fld_fid)
where
  tm_cdb_view_part.sid_corporate=p.sid_corporate and
  tm_cdb_view_part.table_id=p.table_id and
  tm_cdb_view_part.view_id=p.view_id and
  tm_cdb_view_part.id=p.id and
  (v.holder_field_type is null or v.holder_field_type='input_list')
;
